<?php

namespace Drupal\Tests\page_cache_query_ignore\Unit\StackMiddleware;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\PageCache\RequestPolicyInterface;
use Drupal\Core\PageCache\ResponsePolicyInterface;
use Drupal\page_cache_query_ignore\StackMiddleware\PageCacheIgnore;
use Drupal\Tests\UnitTestCase;
use Symfony\Component\HttpKernel\HttpKernelInterface;

/**
 * @coversDefaultClass \Drupal\page_cache_query_ignore\StackMiddleware\PageCacheIgnore
 * @group page_cache_query_ignore
 */
class PageCacheIgnoreTest extends UnitTestCase {

  /**
   * Page cache ignore object configured on setUp().
   *
   * @var \Drupal\page_cache_query_ignore\StackMiddleware\PageCacheIgnore
   */
  protected $pageCacheIgnore;

  /**
   * {@inheritdoc}
   */
  public function setUp() {
    parent::setUp();

    $http_kernel = $this->getMockBuilder(HttpKernelInterface::class)
      ->getMock();
    $cache = $this->getMockBuilder(CacheBackendInterface::class)
      ->getMock();
    $request_policy = $this->getMockBuilder(RequestPolicyInterface::class)
      ->getMock();
    $response_policy = $this->getMockBuilder(ResponsePolicyInterface::class)
      ->getMock();
    $config_factory = $this->getConfigFactoryStub([
      'page_cache_query_ignore.settings' => [
        'ignored_parameters' => ['gclid', 'msclid', 's'],
      ],
    ]);

    $this->pageCacheIgnore = new PageCacheIgnore($http_kernel, $cache, $request_policy, $response_policy, $config_factory);
  }

  /**
   * Test the ignored parameters are loading correctly.
   *
   * @covers ::getIgnoredParams
   */
  public function testGetIgnoredParams() {
    $reflection = new \ReflectionClass(PageCacheIgnore::class);
    $method = $reflection->getMethod('getIgnoredParams');
    $method->setAccessible(TRUE);

    $this->assertArrayEquals(['gclid', 'msclid', 's'], $method->invoke($this->pageCacheIgnore), "PageCacheIgnore::getIgnoredParams matched settings");
  }

  /**
   * Test the cleared URLs are working as expected.
   *
   * @covers ::clear
   *
   * @dataProvider clearProvider
   */
  public function testClear($input, $expected) {
    $reflection = new \ReflectionClass(PageCacheIgnore::class);
    $method = $reflection->getMethod('clear');
    $method->setAccessible(TRUE);

    $this->assertEquals($expected, $method->invokeArgs($this->pageCacheIgnore, [$input]), 'PageCacheIgnore::clear removed correct parameters');
  }

  /**
   * Provides test data for testClear().
   */
  public function clearProvider() {
    return [
      ['url/', 'url/'],
      ['url/?gclid=my-value', 'url/'],
      ['url/?a=abc&gclid=my-value', 'url/?a=abc'],
      ['url/?a=abc&gclid=my-value&b=def', 'url/?a=abc&b=def'],
      ['url/?a=abc&gclid=my-value&msclid=my-value2&b=def', 'url/?a=abc&b=def'],
      ['url/gclid', 'url/gclid'],
      ['url/?a=gclid&msclid=def', 'url/?a=gclid'],
      ['url/keys?keys=abc&s=123', 'url/keys?keys=abc'],
      ['url/?s[]=abc&s[]=def&s[]=ghi', 'url/'],
    ];
  }

}
